*Set folder/working directory paths:
global rawdata_folder			  [paste folder path here without quotation marks]
global intermediate_data_folder   [paste folder path here without quotation marks]
global final_data_folder		  [paste folder path here without quotation marks]
global dofiles_folder			  [paste folder path here without quotation marks]

STOP // When working with the code, this line is useful to avoid accidentally running the whole do-file from the top.


*Log
cd "$final_data_folder"
log using "Generate industry skill relatedness data.log", replace

/*

I den här do-filen bygger vi flera dataset som fångar så kallad skill relatedness mellan branscher.
Med andra ord försöker vi mäta hur nära besläktade branscher är med avseende på det humankapital som de använder
baserat på hur många personer som byter jobb från den ena branschen till den andra.
Metodiken förklaras i tillhörande PM med titeln "Metod och data för att beräkna skill relatedness mellan industrier baserat på svenska data"
och är baserad på Neffke, Otto & Weyh (2017).
Vilken bransch varje individ jobbar i definieras av indelningen SNI 2007 på arbetsställenivå. 

This do-file uses Swedish linked employer-employee registry data (LEED) to generate matrices (datasets) 
which contain information about skill relatedness between industries based on labor flows. 
The industry classification used is NACE Rev. 2. The methodology is explained (in Swedish) in the memo titled 
"Metod och data för att beräkna skill relatedness mellan industrier baserat på svenska data"
and is closely based on Neffke, Otto & Weyh (2017), "Inter-industry labor flows".

The do-file uses the following data sources:
- FAD registry (FAD ID and other variables for firms and plants)
- LISA registry (data on individuals: person ID, firm and plant IDs, workplace/plant industry code, income, education, employment status)
- KCR registry (corporate group ID for firms)
- Datasets with all industry codes
It also uses two do-files which create the industry labels.

By Erik Engberg, policy analyst at Growth Analysis, 2020.

CONTENTS:
1. CREATE PANEL WITH ALL EMPLOYED INDIVIDUALS
2. KEEP ONLY JOB SWITCHERS AND COLLAPSE TO YEARLY MATRICES
3. CALCULATE RELATEDNESS AND GENERATE FINAL MATRICES
4. GENERATE DATA SETS SUITABLE FOR DRAWING NETWORK GRAPHS
5. DESCRIPTIVE ANALYSIS OF THE RELATEDNESS MEASURES

*/



*****************************************************
*** 1. CREATE PANEL WITH ALL EMPLOYED INDIVIDUALS ***
*****************************************************

*** 1A. APPEND YEARLY LISA TABLES ***

*Create an empty dataset which we can append data into
clear
set obs 0
gen year=.
gen personID=.
gen firmID=.
gen plantID=.
gen NACE=""
gen income=.
gen education_level=""
gen employed=.

cd "$rawdata_folder"
save workers_panel_2007_2017, replace

*Append data for the ten years, 2008-2017,
*to create a yearly panel with the entire population of Swedish individuals ages 16-84,
*including the NACE code of the plant where they worked in the year,
*according to the RAMS employer-employee link, which is included in the LISA registry.
forvalues year = 2007/2017 {

	odbc load , table("dbo.Individ_`year'") clear noquote

	rename AstSNI2007 NACE
	rename P1043_LopNr_PersonNr personID
	rename P1043_LopNr_PeOrgNr_LISA firmID
	rename P1043_LopNr_CfarNr_LISA plantID
	rename Sun2000niva education_level
	rename LoneInk income
	capture rename SyssStatJ SyssStat
	capture rename SyssStat11 SyssStat
	gen year = `year'
	capture destring SyssStat, replace
	capture destring income, replace
	
	keep year personID firmID plantID NACE SyssStat education_level income
	
		*The dataset includes the entire population above the age of 16.
		noisily count
		
		*SyssStat indicates the individual's "employment status" in November,
		*where 1=fully employed, 5=partially employed and 6=not employed.
		*Not employed can mean both unemployed and not part of the labor force (e.g. retirees, disabled).
		noisily tab SyssStat
		
		*We have plant NACE code for all but 1,15 % of the employed.
		*Note: this variable changed in 2011.
		noisily tab NACE if SyssStat==1
		
		*Hmm, for this employment category we have plant NACE codes for all but 12 %.
		noisily tab NACE if SyssStat==5
		
		*We have no NACE codes for those who are not employed.
		*The NACE variable is equal to "00000" for these.
		noisily tab NACE if SyssStat==6
		
	*Let's use the NACE codes only for those individuals who were fully employed (according to the RAMS definition)
	replace NACE="00000" if SyssStat==5
	
	gen 	employed=0
	replace employed=1 if SyssStat==1
	
	drop SyssStat
	
	duplicates drop personID, force
	tsset personID year
	
	cd "$rawdata_folder"
	append using workers_panel_2007_2017
	
	cd "$rawdata_folder"
	save workers_panel_2007_2017, replace

}	




*** 1B. ADD FIRM AND PLANT IDs FROM FAD, AS WELL AS CORPORATE GROUP ID ***

*Following Neffke & Henning (2013), we want to count only those workers who are genuinely switching jobs.
*For example, we do not want to count those who are reassigned to another plant within the same firm.
*We have already included the plant/establishment and firm/organization IDs in the panel.
*In order to further enhance the quality of our measure of genuine job switching, 
*we add firm and plant IDs from the FAD ("dynamics of firms and establishments") registry,
*which helps us to eliminate cases where firms or plants switch IDs.
*We also add corporate/enterprise group IDs.

*Load the firm-level FAD
cd "$rawdata_folder"	
use o_for_fadf_1990_2017, clear	

		count
	
	rename ar2 year
	rename peorglopnr2 firmID
	rename peorglopnr1 firmID_prev_FAD
	rename t2 firm_size
	keep year firmID firm_size kvar firmID_prev_FAD
	keep if year>=2007 & year<=2017
		
cd "$rawdata_folder"
merge 1:m firmID year using workers_panel_2007_2017

	*100 % of individuals tagged as fully employed in LISA/RAMS found a matching firm in the FAD registry.
	tab _merge if employed==1
	*There were no individuals whose firms were observed in FAD but not LISA.

drop _merge
	
cd "$rawdata_folder"
save workers_panel_2007_2017, replace



*Plant-level FAD
*Begin by creating a panel with the FAD data
	clear
	set obs 0
	gen year=.
	gen plantID=.
	gen plantID_FAD=.
	
	cd "$rawdata_folder"
	save plant_FAD_panel_2007_2017, replace
	
	forvalues year = 2007/2017 {
	
		odbc load, table("dbo.FAD_ARBETSSTALLE`year'") clear noquote
		
		gen year = `year'
		keep year P1043_LopNr_CfarNr2 P1043_LopNr_FAD_A_ID
		rename P1043_LopNr_CfarNr2 plantID
		rename P1043_LopNr_FAD_A_ID plantID_FAD
		order year plantID plantID_FAD
		
		*Drop plants that lack plant ID
		drop if plantID==0
		
		tsset plantID year
		
			count
		
		cd "$rawdata_folder"
		append using plant_FAD_panel_2007_2017
		
		cd "$rawdata_folder"
		save plant_FAD_panel_2007_2017, replace
	
	}

*Now merge the FAD-panel with the big worker panel
	cd "$rawdata_folder"
	use plant_FAD_panel_2007_2017, clear

	cd "$rawdata_folder"
	merge 1:m plantID year using workers_panel_2007_2017
	
	*There were only a handful of plants that didn't find any workers in LISA.
	*Drop these.
	drop if _merge==1
	drop _merge
	
	cd "$rawdata_folder"
	save workers_panel_2007_2017, replace

	

*Add enterprise/corporate group ID
odbc load, table("dbo._FDB_KCR_1999_2017") clear noquote
	
	keep ar P1043_LopNr_PeOrgNr Lopnr_peorgnrkoncmod
	rename ar year
	rename P1043_LopNr_PeOrgNr firmID
	rename Lopnr_peorgnrkoncmod groupID
	destring year firmID, replace
	keep if year>=2007
	
	*We have a problem here: in this version of the corporate group database, 
	*20 % of group IDs begin with the letter K.
	*I have never seen this before when working with group data and don't know what to make of it...
	*We will simply drop those 20 %. Adjusting the job switcher data for corporate group 
	*barely affects the end result anyway, so the impact of this will be negligible.
	gen K_in_groupID = strpos(groupID,"K")
	drop if K_in_groupID > 0
	drop K_in_groupID
	*We can now destring the group ID variable.
	destring groupID, replace
	
		count
	
	cd "$rawdata_folder"
	merge 1:m firmID year using workers_panel_2007_2017
	
	*We have more unmatched firms this time. These are most likely mostly firms that did not employ 
	*one full-time worker and are therefore not observed in LISA.
	drop if _merge==1
	drop _merge
	
	*For firms that were not part of a corporate group,
	*let the corporate group ID be replaced by the firm ID.
	replace groupID = firmID if groupID==. & firmID!=. & firmID!=0
	
	*Finishing touches:

	*Add variable labels.
	label var year "Year"
	label var personID "Person ID (perslopnr)"
	label var firmID "Firm/organization ID (peorglopnr)"
	label var firmID_prev_FAD "Firm ID in previous year according to FAD registry"
	label var kvar "FAD code for firm status compared to previous year"
	label var firm_size "Firm size, from FAD (t2)"
	label var plantID "Plant/establishment ID (cfarlopnr)"
	label var plantID_FAD "Plant ID according to FAD registry"
	label var groupID "Enterprise group ID (peorgnrkmlopnr)"
	label var NACE "Plant NACE Rev. 2 industry code (SNI 2007)"
	label var income "Labor income (bruttolöneinkomst, LoneInk)"
	label var education_level "Educational level (SUN2000Niva, 3-digit)"
	label var employed "Employed full time (RAMS definition, November. SyssStat=1)"

	destring education_level NACE, replace
	compress
	order year personID firmID plantID plantID_FAD groupID NACE employed education_level income
	
	*We have now completed the panel including all variables which we will be needing to calculate relatedness.
	cd "$rawdata_folder"
	save workers_panel_2007_2017, replace



******************************************************************
*** 2. KEEP ONLY JOB SWITCHERS AND COLLAPSE TO YEARLY MATRICES ***
******************************************************************

*In this section, we will start with the panel of workers,
*remove irrelevant/unwanted observations, and then collapse the panel
*to a dataset of yearly matrices (in long form) showing the worker flows between industries each year.
*Our approach is to keep only "high quality" observations, i.e. for which we have full information
*on firm and plant of employment. Because the registry is of such high quality overall,
*this cleaning does not cause a dramatic loss of data.
*And because we have so many observations, we can afford to sacrifice N for quality.

cd "$rawdata_folder"
use workers_panel_2007_2017, clear

	count

*Keep only the full time employed
*(reduces the no. of observations by about 40 %)
keep if employed==1

*Drop observations without NACE code
*(we lose another 1 %, approximately)
drop if NACE==0 | NACE==.

*Drop observations without firm ID (none, in this version)
drop if firmID==. | firmID==0

*Drop observations without plant ID (about 5 % of observations)
drop if plantID==. | plantID==0

*Drop firms that are not observed in FAD (we have no zeroes for these)
*(less than 0.0001 % of observations)
drop if plantID_FAD==. | kvar==.

*Generate dummy indicating that the worker earned above the median for his/her industry.
*(as in Henning & Neffke 2013).
egen median_income_ind = median(income), by(NACE year)
gen 	income_above_median = 0
replace income_above_median = 1 if income>median_income_ind

*Generate dummy indicating that the worker was highly educated (>=2 years of higher education)
*(about one third of the labor force)
gen 	high_edu = 0
replace high_edu = 1 if education_level>=520 & education_level<999

*Carry over information about NACE code of employment in the previous year for each individual.
*From now on we will use the same denomination as has been in used in the literature ... 
*where i refers to the previous year, and j refers to the current year.
tsset personID year
gen NACE_i = L.NACE
rename NACE NACE_j

*Drop observations of individuals that were not employed in the previous year.
*(either because of unemployment, or because they were not part of the labor force,
*e.g. recent graduates and immigrants)
*This reduces the no. of observations by 17 %.
drop if NACE_i==.

*Now, we keep only individuals who worked in different NACE codes in year i and year j.
*This drops 87 % of observations and leaves us with 5,1 million observations of workers switching industries.
keep if NACE_j!=NACE_i
	count
	
*Now, we drop workers whose plant NACE code changes,
*even though they remain within the same firm, plant and/or corporate group.	
drop if firmID==L.firmID // 218 872
drop if firmID!=L.firmID & L.firmID==firmID_prev_FAD & firm_size>5 // 21 325; different firmID but same firm according to FAD, conditional on at least 6 employees because FAD is not very reliable for the smallest firms.
drop if plantID==L.plantID // 1 169
drop if plantID_FAD==L.plantID_FAD // 14 431
drop if groupID==L.groupID // 16 478
*All in all, these steps reduced the no. of observations by 272 263, or about 5 %.

*Having cleaned the job switcher data, we can now collapse the panel to a series of matrices,
*one for each year, showing the flows of workers between industries.
*The matrices are in "long" format, with one observation per flow.
gen one=1
collapse (sum) one high_edu income_above_median, by(NACE_i NACE_j year)

rename one workers
rename high_edu workers_hi_edu
rename income_above_median workers_hi_income

	label var year "Year (j)"
	label var NACE_j "Industry which workers switched to"
	label var NACE_i "Industry which workers switched from"
	label var workers "No. of workers switching from industry i to industry j"
	label var workers_hi_edu "Workers with two or more years of higher education"
	label var workers_hi_income "Workers with income above median for industry (and year)"

*Add NACE codes on all levels	
	
	*Add all NACE levels for year 1
		rename NACE_i sni07
		cd "$dofiles_folder"
		do sni2007_polerad
		
		foreach var of varlist sni07_* {
			label var `var' "Industry of employment in year 1 (previous year)"
			rename `var' `var'_i
		}
		drop sni07
		
	*Add all NACE levels for year 2
		rename NACE_j sni07
		cd "$dofiles_folder"
		do sni2007_polerad
		drop sni07
		replace sni07_a     =0 if sni07_a	  ==.
		replace sni07_a_i=0 if sni07_a_i==.
		
		foreach var of varlist sni07_a sni07_2 sni07_3 sni07_4 sni07_5 {
			label var `var' "Industry of employment in year 2 (current year)"
			rename `var' `var'_j
		}
	
	
cd "$intermediate_data_folder"
save worker_flows_matrices_2007_2017, replace



********************************
*** 3. CALCULATE RELATEDNESS ***
********************************

*** 3A. CALCULATE RELATEDNESS FOR THREE TYPES OF WORKERS AND FIVE INDUSTRY LEVELS ***

*We do everything three times for three different types of workers:
*1. all workers, 2. workers with at least two years of higher education, 
*3. workers earning above median income in the industry which they switched to.
*We produce 18 datasets/matrices: three worker types times five NACE levels,
*plus, for each of the three workers types, one data set on the 5-digit NACE level
*which includes all the underlying variables that were used for the relatedness calculation.
foreach worker_type in workers workers_hi_edu workers_hi_income {
	
	cd "$intermediate_data_folder"
	use worker_flows_matrices_2007_2017, clear
		
	*For each of the three types of workers, 
	*we generate relatedness datasets on each NACE (SNI) level.
	foreach industry in sni07_a sni07_2 sni07_3 sni07_4 sni07_5 {
		
		preserve
		
		collapse (sum) `worker_type', by(`industry'_i `industry'_j)
		
		sort `industry'_j `industry'_i
		
			label var `worker_type' "Number of workers switching from `industry'_i to `industry'_j, 2009-2017"
			
		*Calculate the total number of workers switching to each industry.
		egen inflow_total_j = sum(`worker_type'), by(`industry'_j)
		
		*Calculate the total number of workers switching from each industry.
		egen outflow_total_i = sum(`worker_type'), by(`industry'_i)
		
		*Calculate the total number of workers switching industry (sum of all industries)
		egen flow_total_all = sum(`worker_type')
		
		*Calculate each industry's (i) contribution to the inflows to a given industry (j).	
		gen share_of_switchers_to_j = `worker_type'/inflow_total_j
		
		*Calculate each industry's (i) contribution to the total inflows 
		*summed over all receiving industries (j).
		gen share_of_switchers_to_all = outflow_total_i/flow_total_all
		
		*Predict how many workers "should" be switching from industry i to j,
		*if all workers switched industries at random.
		gen predicted_flow = share_of_switchers_to_all*inflow_total_j
		
		*Calculate relatedness index based on the flow of workers from industry_i to industry_j
		gen relatedness_index = `worker_type'/predicted_flow
		
		*For the 5-digit level only, we save a version of the dataset which includes all the
		*underlying variables that are used in the calculation of relatedness. 
		*The purpose of this is to make it more transparent how the index was calculated, 
		*and to allow for experimentation.
		if "`industry'"=="sni07_5" {
		
			cd "$intermediate_data_folder"
			save temp_file, replace
			
			rename relatedness_index relatedness_index_ij
			gen sr_ij_star = (relatedness_index_ij-1)/(relatedness_index_ij+1)
			
			*Divide the index into a few categories of relatedness.
			gen rel_cat = 0
			replace rel_cat=1 if sr_ij_star>0.1 & sr_ij_star<=0.33
			replace rel_cat=2 if sr_ij_star>0.33 & sr_ij_star<=0.8
			replace rel_cat=3 if sr_ij_star>0.8 & sr_ij_star<=0.95
			replace rel_cat=4 if sr_ij_star>0.95 & sr_ij_star<=1
			
				*capture label define rel_cat_labels 0 "Unrelated" 1 "Weakly related" ///
				*2 "Moderately related" 3 "Strongly related" 4 "Very strongly related"
				*label values rel_cat rel_cat_labels
				
				*The final product needs to be in Swedish. 
				capture label define rel_cat_labels 0 "Orelaterade" 1 "Svagt relaterade" ///
				2 "Måttligt relaterade" 3 "Starkt relaterade" 4 "Mycket starkt relaterade"
				label values rel_cat rel_cat_labels
				
				label var rel_cat "Relatedness category, 0-4"
				label var relatedness_index_ij "Relatedness index based on flows of workers from i to j"
				label var sr_ij_star "Normalized relatedness index, -1 to 1"
				label var `industry'_i "industry where the workers were employed in year 1"
				label var `industry'_j "industry which workers switched to in year 2"
				label var inflow_total_j "Total no. of workers switching from other industries to `industry'_j, 2009-2017"
				label var outflow_total_i "Total no. of workers switching from industry i to other industries"
				label var flow_total_all "Total no. of workers switching industries during 2007-2017"
				label var share_of_switchers_to_j "Industry i's share of the workers switching to industry j"
				label var share_of_switchers_to_all "Industry i's share of all workers switching industries"
				label var predicted_flow "Predicted no. of workers switching from i to j"
			
			cd "$intermediate_data_folder"
			save `industry'_relatedness_`worker_type'_unidrectional_large, replace
			
			cd "$intermediate_data_folder"
			use temp_file, clear
		
		}	
			
		*Drop "flows" consisting of only one worker or no workers.
		drop if `worker_type'<=1	
			
		drop `worker_type' inflow_total_j outflow_total_i flow_total_all share_of_switchers_to_j ///
		share_of_switchers_to_all predicted_flow
		
		*We do the following in order to get the flows in both directions on the same row:
		rename relatedness_index relatedness_index_ij
		
		cd "$intermediate_data_folder"
		save temp_file, replace
		
			rename `industry'_j `industry'_tempname
			rename `industry'_i `industry'_j
			rename `industry'_tempname `industry'_i
			rename relatedness_index_ij relatedness_index_ji
		
			tempfile reverse_flow
			save `reverse_flow', replace
		
		cd "$intermediate_data_folder"
		use temp_file, clear
			
		merge 1:1 `industry'_j `industry'_i using `reverse_flow'	
		
		*Keep only bilateral relationships where there were flows of at least two workers in each direction.
		*We thus set a slightly higher threshold for links between industries to be included at all 
		*in the relatedness measure.
		keep if _merge==3
		drop _merge
		
		*Normalize the index to -1 to 1
		gen sr_ij_star = (relatedness_index_ij-1)/(relatedness_index_ij+1)
		gen sr_ji_star = (relatedness_index_ji-1)/(relatedness_index_ji+1)
		
		*"Symmetrize" the two flows to a composite measure by taking the average of the two
		gen relatedness_index = (sr_ij_star+sr_ji_star)/2	
		
			*Each row now contains flows going in both directions, so we need to label the variables accordingly.
			label var `industry'_j "Industry j"
			label var `industry'_i "Industry i"
			label var relatedness_index_ij "Relatedness based on workers flowing from industry i to industry j"
			label var relatedness_index_ji "Relatedness based on workers flowing from industry j to industry i"
			label var relatedness_index "Relatedness of industry i and industry j (undirected)"
		
		keep `industry'_i `industry'_j relatedness_index
		sort `industry'_i `industry'_j 
		
		*Divide the index into a few categories of relatedness.
		gen     rel_cat_`worker_type'=0
		replace rel_cat_`worker_type'=1 if relatedness_index>0.1  & relatedness_index<=0.33
		replace rel_cat_`worker_type'=2 if relatedness_index>0.33 & relatedness_index<=0.8
		replace rel_cat_`worker_type'=3 if relatedness_index>0.8  & relatedness_index<=0.95
		replace rel_cat_`worker_type'=4 if relatedness_index>0.95 & relatedness_index<=1
		
			capture label define rel_cat_labels 0 "Orelaterade" 1 "Svagt relaterade" ///
			2 "Måttligt relaterade" 3 "Starkt relaterade" 4 "Mycket starkt relaterade"
			label values rel_cat_`worker_type' rel_cat_labels
			label var rel_cat_`worker_type' "Relatedness category, 0-4"
		
		rename relatedness_index rel_index_`worker_type'
		
		*Save this preliminary version of the matrix.
		cd "$intermediate_data_folder"
		save `industry'_relatedness_`worker_type', replace
		
		restore
		
	}
	
}

*Clean up: delete the temporary file which we created in the loop
cd "$intermediate_data_folder"
erase "temp_file.dta"



*** 3B. FOR EACH INDUSTRY LEVEL, COMBINE RELATEDNESS MEASURES BASED ON THE THREE TYPES OF WORKERS ***

*Now, create datasets which include relatedness based on all three types of workers.
foreach industry in sni07_a sni07_2 sni07_3 sni07_4 sni07_5 {

	cd "$intermediate_data_folder"
	use `industry'_relatedness_workers, clear
	
	*Merga in the data on relatedness based on highly educated workers.
	cd "$intermediate_data_folder"
	merge 1:1 `industry'_i `industry'_j using `industry'_relatedness_workers_hi_edu
	
	*Fix relatedness for flows that were observed among all workers
	*but not the highly educated workers.
	replace rel_cat_workers_hi_edu = 0 if rel_cat_workers_hi_edu==.
	replace rel_index_workers_hi_edu = -1 if rel_index_workers_hi_edu==.
	drop _merge
	
	*Merge in the data on high-income workers.
	cd "$intermediate_data_folder"
	merge 1:1 `industry'_i `industry'_j using `industry'_relatedness_workers_hi_income
	
	replace rel_cat_workers_hi_income = 0 if rel_cat_workers_hi_income==.
	replace rel_index_workers_hi_income = -1 if rel_index_workers_hi_income==.
	drop _merge
	
	rename rel_index_workers rel_index
	rename rel_cat_workers rel_cat
	rename rel_index_workers_hi_edu rel_index_hi_edu
	rename rel_cat_workers_hi_edu rel_cat_hi_edu
	rename rel_index_workers_hi_income rel_index_hi_income
	rename rel_cat_workers_hi_income rel_cat_hi_income
	
		label var rel_index "Relatedness index. All workers. Undirected."
		label var rel_cat "Relatedness category, 0-4. All workers."
		label var rel_index_hi_edu "Relatedness index. Workers with >=2 ys university education."
		label var rel_cat_hi_edu "Relatedness category. Workers with >=2 ys university education."
		label var rel_index_hi_income "Relatedness index. Workers with above median income in industry j."
		label var rel_cat_hi_income "Relatedness category. Workers with above median income in industry j."
	
	sort `industry'_j rel_index
	
	cd "$intermediate_data_folder"
	save `industry'_relatedness_combined, replace

}



*** 3C. CREATE MATRICES WITH ALL POSSIBLE INDUSTRY COMBINATIONS AND ADD RELATEDNESS ESTIMATES ***

*For the final result, we want to create a matrix which includes all possible industry combinations,
*not just the ones where worker flows/job switchers are observed in the data. 
*For industry pairs that are both observed in the data, yet have no workers switching between them,
*we can conclude that they are unrelated, based on our data.
*In those cases where one or both of the industries are never observed in the data,
*we cannot draw any conclusions about their relatedness, and hence leave the cell empty ("missing").
*We also define each industry as being perfectly related to itself.
*One matrix is created for each industry level.

foreach industry in sni07_a sni07_2 sni07_3 sni07_4 sni07_5 {

	*Import the full list of NACE codes
	cd "$rawdata_folder"
	use `industry', clear
	
		keep kod
		destring kod, replace
		
			label var kod ""
		
		cd "$rawdata_folder"
		save `industry'_all_codes, replace
	
	
	*Now we can create the "base matrix" with all possible NACE combinations.
	*Begin by creating an empty dataset which we can append into.
	clear
	
		set obs 0
		gen `industry'_i=.
		gen `industry'_j=.
		
		cd "$rawdata_folder"
		save base_matrix_`industry', replace
	
	
	*For each industry j, we create a dataset with one row per possible industry i NACE code,
	*and append it to the matrix.
	cd "$rawdata_folder"
	use `industry'_all_codes, clear
	
		levelsof kod, local(industry_codes)
		foreach industry_code of local industry_codes {
		
			preserve
				
				rename kod `industry'_i
				gen `industry'_j = `industry_code'
				
				cd "$rawdata_folder"
				append using base_matrix_`industry'
				
				cd "$rawdata_folder"
				save base_matrix_`industry', replace
				
			restore
		
	}
	
	
	*Sort the industry combinations in an easy to read order
	cd "$rawdata_folder"
	use base_matrix_`industry', clear
		
		sort `industry'_i `industry'_j
		
		cd "$rawdata_folder"
		save base_matrix_`industry', replace
	
	
	*Now merge the "base matrix" with our previously calculated relatedness variables.
	cd "$rawdata_folder"
	use base_matrix_`industry', clear
	
		cd "$intermediate_data_folder"
		merge 1:1 `industry'_i `industry'_j using `industry'_relatedness_combined
		
		*Value- and variable labels for the industries were lost in the merge, let's put them back:
		*(the labels were stored in the memory earlier, when we ran the do-file "sni2007_polerad")
		label values `industry'_i `industry'_polished_labels
		label values `industry'_j `industry'_polished_labels
		label var `industry'_i "Industry i"
		label var `industry'_j "Industry j"
		
			*34 industry codes out of 821 are not observed in the Swedish data
			*(i.e., either they did not exist at all in Sweden or they did but
			*their worker flows were too small to calculate relatedness with at least one other industry)
			unique `industry'_i
			unique `industry'_i if _merge==3
		
		*For industry combinations where both are observed in our job switching data,
		*i.e. where a relatedness index was estimated at least once for both of the industries,
		*we replace missing values in the base matrix with -1. 
		*For industry combinations where at least one of the industries is not observed in our data,
		*we leave the index as missing, to underscore the fact that we have no data 
		*on the relatedness between those two industries.
		egen max_index_i = max(rel_index), by(`industry'_i)
		egen max_index_j = max(rel_index), by(`industry'_j)
		
		*We have already ensured that the two other measures are equal to -1 
		*in those cases where they are missing (i.e. no switchers are observed), but rel_index is not.
		replace rel_index_hi_edu 	= -1 if max_index_i!=. & max_index_j!=. & rel_index==.
		replace rel_index_hi_income = -1 if max_index_i!=. & max_index_j!=. & rel_index==.
		replace rel_index 			= -1 if max_index_i!=. & max_index_j!=. & rel_index==.
		
		replace rel_cat = 0 if rel_index==-1
		replace rel_cat_hi_edu = 0 if rel_index_hi_edu==-1
		replace rel_cat_hi_income = 0 if rel_index_hi_income==-1
		
		*We also follow Neffke et al. (2017) in defining industries as being perfectly related to themselves.
		*These values are not useful for analysis, but they complete the matrix in a logical way.
		replace rel_index = 1 if `industry'_i==`industry'_j
		replace rel_cat = 4  if rel_index==1
		
		replace rel_index_hi_edu = 1 if `industry'_i==`industry'_j
		replace rel_cat_hi_edu = 4  if rel_index_hi_edu==1
		
		replace rel_index_hi_income = 1 if `industry'_i==`industry'_j
		replace rel_cat_hi_income = 4  if rel_index_hi_income==1
		
		drop _merge max_index_i max_index_j

		*Ideally I should have given the variables Swedish names from the start.
		*In retrospect, I should also have created the categories at the very end.
		*Oh well, it's too much effort to go back and change at this point...
		rename rel_cat rel_kat
		rename rel_cat_hi_edu rel_kat_hogutb	
		rename rel_cat_hi_income rel_kat_hogink
		rename rel_index_hi_edu rel_index_hogutb
		rename rel_index_hi_income rel_index_hogink
		
		*Save a preliminary Stata version of the matrix which includes the categories...
		cd "$intermediate_data_folder"
		save `industry'_relatedness, replace
		
		*For the final Stata version, we drop the categories, round the index to four decimals,
		*And add Swedish variable labels.
		drop rel_kat*
		
		replace rel_index 		 = round(rel_index,0.0001)
		replace rel_index_hogink = round(rel_index_hogink,0.0001)		
		replace rel_index_hogutb = round(rel_index_hogutb,0.0001)		
		
		label var `industry'_i "Bransch i"
		label var `industry'_j "Bransch j"		
		label var rel_index "Relatedness index, samtliga arbetare"
		label var rel_index_hogutb "Relatedness index, arbetare med >=2 år av högskoleutbildning"		
		label var rel_index_hogink "Relatedness index, arbetare med inkomst över branschens median"
		order `industry'_i `industry'_j rel_index rel_index_hogink rel_index_hogutb
				
		*The final version of the matrix in Stata format is now done!
		cd "$final_data_folder"
		save `industry'_relatedness, replace
		
}



*** 3D. CREATE FINAL MATRICES SUITABLE FOR STORAGE IN SQL SERVER (.CSV FORMAT) ***

*As a final step, we create a version of the matrices which is suitable for storage in SQL server.
*We also create one set of matrices with just the basic index based on all workers,
*while creating separate matrices which include indeces based on all three worker types.
*This is in order to make the main matrices as simple as possible, thus reducing confusion.

foreach industry in sni07_a sni07_2 sni07_3 sni07_4 sni07_5 {

cd "$final_data_folder"
use `industry'_relatedness, clear

	*Because it is not, as far as I know, possible to preserve the value labels 
	*when storing the data in SQL server, we create separate columns for the industry labels.
	decode `industry'_i, gen(`industry'_i_text)
	decode `industry'_j, gen(`industry'_j_text)
	label drop _all
	order `industry'_i `industry'_j `industry'_i_text `industry'_j_text rel_index ///
	rel_index_hogutb rel_index_hogink
	
	*We want to round the relatedness index to the fourth decimal.
	*Annoyingly, because of precision issues, this is much harder to do than it might seem. 
	*Stata cannot store certain decimals exactly,
	*e.g. 0.8572 becomes .8572000000000001 when imported to Excel.
	*The convuluted/ugly solution i came up with involves saving the index variable as a string...
	foreach index_version in rel_index rel_index_hogutb rel_index_hogink {
		
		recast double `index_version'
	
		gen 	index2  = round(`index_version',0.0001)
		replace index2 = index2+0.000000001 if index2>0  & index2<1
		replace index2 = index2-0.000000001 if index2>-1 & index2<0
		tostring index2, gen(index3) force
		
		gen 	index4=""
		replace index4=index3 if inlist(index2,-1,0,1)
		replace index4=substr(index3,1,5) if index2>0 & index2<1
		replace index4=substr(index3,1,6) if index2<0 & index2>-1
		
		drop `index_version' index2 index3
		rename index4 `index_version'
		
	}

	*Finally, we export the matrix to .csv format.
	cd "$final_data_folder"
	export delimited using "`industry'_relatedness", delimiter(tab) replace
	
}



*** 3E. Prepare the "extended" matrix for SQL storage ***

cd "$intermediate_data_folder"
use sni07_5_relatedness_workers_unidrectional_large, clear

	decode sni07_5_i, gen(sni07_5_i_text)
	decode sni07_5_j, gen(sni07_5_j_text)
	rename rel_cat rel_kat
	decode rel_kat, gen(rel_kat_text)
	label drop _all
	
	rename workers arbetare
	rename inflow_total_j inflode_totalt_j
	rename outflow_total_i utflode_totalt_i
	rename flow_total_all totalt_flode_alla
	rename share_of_switchers_to_j andel_av_jobbytare_till_j
	rename share_of_switchers_to_all andel_av_alla_jobbytare
	rename predicted_flow predicerat_flode
	
	order sni07_5_i sni07_5_j sni07_5_i_text sni07_5_j_text 
	
	*Save in Stata format
	cd "$final_data_folder"
	save sni07_5_relatedness_utokad, replace
	
	*Export as .csv
	cd "$final_data_folder"
	export delimited using "sni07_5_relatedness_utokad", delimiter(tab) replace
	
	

*****************************************************************
*** 4. GENERATE DATA SETS SUITABLE FOR DRAWING NETWORK GRAPHS ***
*****************************************************************

*** 4A. GENERATE LINK/EDGE DATASETS FOR USE IN GEPHI ***

*Following Neffke et al (2017), we keep the 3N strongest links,
*where N is the observed number of industries.
foreach industry in sni07_a sni07_2 sni07_3 sni07_4 sni07_5 {

	*The dataset in the intermediate folder contains exactly the same index-data,
	*except it also retains the categories that were dropped for the "official" version.
	cd "$intermediate_data_folder"
	use `industry'_relatedness, clear	
	
	drop if `industry'_j>=`industry'_i
	drop if rel_index==.
	unique `industry'_j
	scalar threeN = r(sum)*3
	sort rel_index
	drop if _n<=_N-threeN
	
	cd "$intermediate_data_folder"
	save `industry'_edges_3N, replace

	gen Source=`industry'_i 
	gen Target=`industry'_j 
	
	*Weight links according to the relatedness strength category
	*We add three so that the weights are not so dramatically different.
	gen rel_kat_2=rel_kat+3
	
	cd "$intermediate_data_folder"
	export excel using "Industry edges 3N.xls", sheet("`industry'")  firstrow(variables) sheetmodify

	}



*** 4B. GENERATE NODE DATASETS ***

*Begin by collapsing the big 2007-2017 panel with individuals to one observation per industry,
*summing up the share of the economy's employment in that industry during the period.
cd "$rawdata_folder"
use workers_panel_2007_2017, clear
	
	keep if employed==1
	destring NACE, replace
	drop if NACE==0
	
	collapse (sum) employed, by(NACE)
	
		rename employed workers
		egen employment_total = sum(workers)
		gen employment_share = workers/employment_total
		
		*Add NACE codes on all levels
		rename NACE sni07
		cd "$dofiles_folder"
		do sni2007_polerad
		drop sni07
		
		*Add coarse sectors with 9 categories
		cd "$dofiles_folder"
		do sni2007_9kat		
		
		gen ID = sni07_5
		
		cd "$intermediate_data_folder"
		export excel using "Industry nodes.xls", sheet("sni07_5")  firstrow(variables) replace
		
		cd "$intermediate_data_folder"
		save nodes_sni07_5, replace		
		
		
*Now produce the same data sets for the other NACE lavels
cd "$intermediate_data_folder"
use nodes_sni07_5, clear
	
	foreach industry in sni07_a sni07_2 sni07_3 sni07_4 {
	
		preserve
		
		rename `industry' temp_`industry'
		
		collapse (sum) employment_share workers (first) sni*, by(temp_`industry')
		
			rename temp_`industry' `industry'
			
			capture label values sni07_3 sni07_3_polished_labels
			capture label values sni07_2 sni07_2_polished_labels
			capture label values sni07_a sni07_a_polished_labels
			capture label values sni_9kat sni_9kat_labels
			
			*Drop industries on lower levels of aggregation
			if "`industry'"=="sni07_4" { 
				drop sni07_5 
			}
			if "`industry'"=="sni07_3" { 
				drop sni07_5 sni07_4 
			}
			if "`industry'"=="sni07_2" { 
				drop sni07_5 sni07_4 sni07_3 
			}
			if "`industry'"=="sni07_a" { 
				drop sni07_5 sni07_4 sni07_3 sni07_2 
			}
			
			gen ID = `industry'
			
			cd "$intermediate_data_folder"
			export excel using "Industry nodes.xls", sheet("`industry'")  firstrow(variables) sheetmodify
			
			cd "$intermediate_data_folder"
			save nodes_`industry', replace
	
		restore
		
	}
	
	
*Now, keep only nodes with strong connections and save in Excel
*(so that we don't get any nodes without links in the graph)
foreach industry in sni07_a sni07_2 sni07_3 sni07_4 sni07_5 {

cd "$intermediate_data_folder"
use nodes_`industry', clear	
	
	rename `industry' `industry'_i
	cd "$intermediate_data_folder"
	merge 1:m `industry'_i using `industry'_edges_3N
	gen matched_`industry'_i = 0
	replace matched_`industry'_i = 1 if _merge==3
	*Drop the merged variables and duplicates that were created in the merge
	drop `industry'_j-_merge
	duplicates drop `industry'_i, force
	
	rename `industry'_i `industry'_j
	cd "$intermediate_data_folder"
	merge 1:m `industry'_j using `industry'_edges_3N
	gen matched_`industry'_j = 0
	replace matched_`industry'_j = 1 if _merge==3
	drop `industry'_i-_merge
	duplicates drop `industry'_j, force
	
	*We have 51 nodes that did not have any edges/links among the 3N strongest.
	count if matched_`industry'_i==0 & matched_`industry'_j==0
	drop  if matched_`industry'_i==0 & matched_`industry'_j==0
	drop matched_`industry'_i matched_`industry'_j
	unique ID
	rename `industry'_j `industry'
	
	*For the 5- and 4-digit levels,
	*create an abbreviated (code only) industry label which is observed only 
	*for the biggest 1/3 of industries
	if _N>400 {
		sort employment_share
		scalar top_third = _N-(_N/3)
		gen Label_top_third = .
		replace Label_top_third=`industry' if _n>top_third
	}
	
	*For the section level and the 2- and 3-digit levels,
	*we create labels for all observations.
	if _N<400 {
		sort employment_share
		gen Label_top_third = .
		replace Label_top_third=`industry'
	}
	
	cd "$intermediate_data_folder"
	export excel using "Industry nodes 3N.xls", sheet("`industry'")  firstrow(variables) sheetmodify	
		
}



***********************************************************
*** 5. DESCRIPTIVE ANALYSIS OF THE RELATEDNESS MEASURES ***
***********************************************************

*For a few chosen industries,
*create a table showing the top industries
*with which it is most related.
*We include the following three: 
*advertising, veterinary activity and manufacture of drugs.
foreach industry in 73111 75000 21200 {

cd "$final_data_folder"
import delimited "sni07_5_relatedness.csv", delimiter(tab) encoding(UTF-8) clear 	
	
	keep if sni07_5_i==`industry'
	keep sni07_5_i_text sni07_5_j_text rel_index rel_index_hogink rel_index_hogutb
	drop if rel_index==.
	drop if rel_index==1
	drop if rel_index==-1
	gen neg_index = rel_index*-1
	sort neg_index
	gen rank=_n
	drop neg_index
	
	cd "$final_data_folder"
	export excel using "Fallstudier av industrier, topp länkar.xls", sheet("`industry'") firstrow(variables) sheetmodify

}


*Tabulate the no. of links in each category of relatedness,
*for each NACE level.
foreach industry in sni07_a sni07_2 sni07_3 sni07_4 sni07_5 {

	cd "$intermediate_data_folder"
	use `industry'_relatedness, clear	
	
	noisily display "`industry'"
	tab rel_kat
	tab rel_kat if rel_kat!=0 & `industry'_i!=`industry'_j
	noisily count
	noisily count if rel_index!=.
	
	preserve
		gen antal_obs = 1
		*Vi vill inte räkna med industriers relationer med
		*sig själva.
		drop if `industry'_i==`industry'_j
		collapse (sum) antal_obs, by(rel_kat)
		egen tot_obs=sum(antal_obs)
		gen andel_obs = antal_obs/tot_obs
		list
	restore
	
}


*For the 5-digit level, create a histogram showing how the relatedness index is distributed
cd "$final_data_folder"
use sni07_5_relatedness, clear
	
	hist rel_index if rel_index>-1 & rel_index<1,  xtitle("Relatedness index") xlabel(-1(0.1)1)

	cd "$final_data_folder"
	graph export "Index histogram.png",  as(png) replace



log close
